/***************************************************************************
 *   Copyright (C) 2003 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the osgGtk library.                              *
 *                                                                         *
 *   The osgGtk library is free software; you can redistribute it and/or   *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The osgGtk library is distributed in the hope that it will be         *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef OSGVIEWER_GRAPHICSWINDOWGTKMM_H
#define OSGVIEWER_GRAPHICSWINDOWGTKMM_H

#include <gtkglmm.h>
#include <osgViewer/GraphicsWindow>

namespace osgViewer
{
  /**
   * This class extends osgViewer::GraphicsWindow to provide an OpenGL rending
   * context for osgViewer::Viewer inside Gtk::GL::DrawingArea
   *
   * @todo add a constructor that takes a traits argument and configures GL mode from traits
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class GraphicsWindowGtkmm: public Gtk::GL::DrawingArea, public osgViewer::GraphicsWindow
  {
    public:
      
      /** Constructs the gtk graphics window of the specified width and height */
      GraphicsWindowGtkmm(int width, int height, Gdk::GL::ConfigMode mode=Gdk::GL::MODE_RGB|Gdk::GL::MODE_DEPTH|Gdk::GL::MODE_DOUBLE);

      virtual ~GraphicsWindowGtkmm();

      virtual bool isSameKindAs(const osg::Object* object) const;
      
      virtual const char* libraryName() const;

      virtual const char* className() const;

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual void grabFocus();

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual void grabFocusIfPointerInWindow();

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool valid() const;

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool setWindowRectangleImplementation(int x, int y, int width, int height);

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual void setCursor(MouseCursor mouseCursor);

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool setWindowDecorationImplementation(bool flag);

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual void setWindowName(const std::string& name);
      
      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool realizeImplementation();

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool isRealizedImplementation() const;

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool makeCurrentImplementation();
      
      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual bool releaseContextImplementation();

      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual void closeImplementation();
      
      /** Overrides osgViewer::GraphicsWindow with Gtkmm specific behavior */
      virtual void swapBuffersImplementation();

      /**
       * Convenience method to swap the OpenGL buffers if double-buffered
       * or call glFlush() if single-buffered
       */
      bool swap_buffers();

      /** Convenience method to implement glBegin() on this window's OpenGL drawable */
      bool gl_begin();

      /** Convenience method to implement glEnd() on this window's OpenGL drawable */
      bool gl_end();

    protected:
      Glib::RefPtr<Gdk::GL::Config> m_gl_config;

      virtual bool on_button_press_event(GdkEventButton* event);

      virtual bool on_button_release_event(GdkEventButton* event);

      virtual bool on_configure_event(GdkEventConfigure* event);

      virtual bool on_key_press_event(GdkEventKey* event);

      virtual bool on_key_release_event(GdkEventKey* event);

      virtual bool on_motion_notify_event(GdkEventMotion* event);

      virtual bool on_proximity_in_event(GdkEventProximity* event);

      virtual bool on_proximity_out_event(GdkEventProximity* event);

      virtual bool on_scroll_event(GdkEventScroll* event);

      void transform_mouse_x_y( double& x, double& y );

  };

}

#endif
